local img = {}

local W = 256
local H = 256
local R = 15

for i = 1, W do
  for j = 1, H do
    img[i * H + j] = 0.5
  end
end

local function blur_lua(img)
  local new_img = {}
  for i = R+1, W-R-1 do
    for j = R+1, H-R-1 do
      local sum = 0
      for rx = i-R, i+R do
        for ry = j-R, j+R do
          sum = sum + img[rx * H + ry]
        end
      end
      new_img[i * H + j] = sum / (R * R)
    end
  end
  return new_img
end

local C = terralib.includec("stdlib.h")
local stdio = terralib.includec("stdio.h")
local blur_terra = terra(img : double[W * H])
  var new_img : &double = [&double](C.malloc(sizeof(double) * W * H))
  for i = R+1, W-R-1 do
    for j = R+1, H-R-1 do
      var sum = 0
      for rx = i-R, i+R do
        for ry = j-R, j+R do
          sum = sum + img[rx * H + ry]
        end
      end
      new_img[i * H + j] = sum / (R * R)
    end
  end
end
blur_terra:compile()

local timg = (terra()
              return C.malloc(sizeof(double) * W * H)
              end)()
local function bench(f, name)
  local start = os.clock()
  if name == 'terra' then
    f(timg)
  else
    f(img)
  end
  print(name, string.format('%.5f', os.clock() - start))
end

bench(blur_lua, 'lua')
bench(blur_terra, 'terra')
