local function make_pow_normal(n)
  return function(x)
    local y = 1
    for i = 1, n do
      y = y * x
    end
    return y
  end
end

local function make_pow_meta(n)
  local s = "return function(x) return "
  for i = 1, n do
    s = s .. "x*"
  end
  s = s .. "1.0 end"
  return loadstring(s)()
end

local function make_pow_terra(n)
  local function make_fn(x)
    local expr = `1.0
    for i = 1, n do
      expr = `[expr] * x
    end
    return expr
  end

  return terra(x:double)
    return [make_fn(x)]
  end
end

local bench = function(f, name)
  local pow500 = f(500)
  if name == 'terra' then
    pow500:compile()
  end
  local start = os.clock()
  for i = 1, 100000 do
    pow500(2.5)
  end
  print(name, string.format('%.5f', os.clock() - start))
end

bench(make_pow_normal, 'normal')
bench(make_pow_meta, 'meta')
bench(make_pow_terra, 'terra')
